@@ -13,10 +13,11 @@
 import logging
 import os.path
 
-from reno import scanner
-
+import six
 import yaml
 
+from reno import scanner
+
 LOG = logging.getLogger(__name__)
 
 
@@ -27,9 +28,7 @@ def get_cache_filename(reporoot, notesdir):
 class Loader(object):
     "Load the release notes for a given repository."
 
-    def __init__(self, reporoot, notesdir, branch=None,
-                 collapse_pre_releases=True,
-                 earliest_version=None,
+    def __init__(self, conf,
                  ignore_cache=False):
         """Initialize a Loader.
 
@@ -38,31 +37,25 @@ class Loader(object):
         Notes files are associated with the earliest version for which
         they were available, regardless of whether they changed later.
 
-        :param reporoot: Path to the root of the git repository.
-        :type reporoot: str
-        :param notesdir: The directory under *reporoot* with the release notes.
-        :type notesdir: str
-        :param branch: The name of the branch to scan. Defaults to current.
-        :type branch: str
-        :param collapse_pre_releases: When true, merge pre-release versions
-            into the final release, if it is present.
-        :type collapse_pre_releases: bool
-        :param earliest_version: The oldest version to include.
-        :type earliest_version: str
+        :param conf: Parsed configuration from file
+        :type conf: reno.config.Config
         :param ignore_cache: Do not load a cache file if it is present.
         :type ignore_cache: bool
-
         """
-        self._reporoot = reporoot
-        self._notesdir = notesdir
-        self._branch = branch
-        self._collapse_pre_releases = collapse_pre_releases
-        self._earliest_version = earliest_version
+        self._config = conf
         self._ignore_cache = ignore_cache
 
+        self._reporoot = conf.reporoot
+        self._notespath = conf.notespath
+        self._branch = conf.branch
+        self._collapse_pre_releases = conf.collapse_pre_releases
+        self._earliest_version = conf.earliest_version
+
         self._cache = None
+        self._scanner = None
         self._scanner_output = None
-        self._cache_filename = get_cache_filename(reporoot, notesdir)
+        self._cache_filename = get_cache_filename(self._reporoot,
+                                                  self._notespath)
 
         self._load_data()
 
@@ -83,13 +76,8 @@ class Loader(object):
                     for n in self._cache['notes']
                 }
         else:
-            self._scanner_output = scanner.get_notes_by_version(
-                reporoot=self._reporoot,
-                notesdir=self._notesdir,
-                branch=self._branch,
-                collapse_pre_releases=self._collapse_pre_releases,
-                earliest_version=self._earliest_version,
-            )
+            self._scanner = scanner.Scanner(self._config)
+            self._scanner_output = self._scanner.get_notes_by_version()
 
     @property
     def versions(self):
@@ -101,9 +89,44 @@ class Loader(object):
         return self._scanner_output[version]
 
     def parse_note_file(self, filename, sha):
-        "Return the data structure encoded in the note file."
+        """Return the data structure encoded in the note file.
+
+        Emit warnings for content that does not look valid in some
+        way, but return it anyway for backwards-compatibility.
+
+        """
         if self._cache:
-            return self._cache['file-contents'][filename]
+            content = self._cache['file-contents'][filename]
         else:
-            body = scanner.get_file_at_commit(self._reporoot, filename, sha)
-            return yaml.safe_load(body)
+            body = self._scanner.get_file_at_commit(filename, sha)
+            content = yaml.safe_load(body)
+
+        for section_name, section_content in content.items():
+            if section_name == 'prelude':
+                if not isinstance(section_content, six.string_types):
+                    LOG.warning(
+                        ('The prelude section of %s '
+                         'does not parse as a single string. '
+                         'Is the YAML input escaped properly?') %
+                        filename,
+                    )
+            else:
+                if not isinstance(section_content, list):
+                    LOG.warning(
+                        ('The %s section of %s '
+                         'does not parse as a list of strings. '
+                         'Is the YAML input escaped properly?') % (
+                             section_name, filename),
+                    )
+                else:
+                    for item in section_content:
+                        if not isinstance(item, six.string_types):
+                            LOG.warning(
+                                ('The item %r in the %s section of %s '
+                                 'parses as a %s instead of a string. '
+                                 'Is the YAML input escaped properly?'
+                                 ) % (item, section_name,
+                                      filename, type(item)),
+                            )
+
+        return content
